@@ -10,9 +10,11 @@ from sqlalchemy.ext.declarative import declared_attr
 from sqlalchemy.ext.declarative import has_inherited_table
 from sqlalchemy.orm import class_mapper
 from sqlalchemy.orm import clear_mappers
+from sqlalchemy.orm import close_all_sessions
 from sqlalchemy.orm import configure_mappers
 from sqlalchemy.orm import create_session
 from sqlalchemy.orm import deferred
+from sqlalchemy.orm import exc as orm_exc
 from sqlalchemy.orm import mapper
 from sqlalchemy.orm import polymorphic_union
 from sqlalchemy.orm import relationship
@@ -29,7 +31,6 @@ from sqlalchemy.testing.schema import Column
 from sqlalchemy.testing.schema import Table
 from test.orm.test_events import _RemoveListeners
 
-
 Base = None
 
 
@@ -39,7 +40,7 @@ class DeclarativeTestBase(fixtures.TestBase, testing.AssertsExecutionResults):
         Base = decl.declarative_base(testing.db)
 
     def teardown(self):
-        Session.close_all()
+        close_all_sessions()
         clear_mappers()
         Base.metadata.drop_all()
 
@@ -1316,7 +1317,9 @@ class OverlapColPrecedenceTest(DeclarativeTestBase):
         self._run_test(Engineer, "eid", "pid")
 
 
-class ConcreteInhTest(_RemoveListeners, DeclarativeTestBase):
+class ConcreteInhTest(
+    _RemoveListeners, DeclarativeTestBase, testing.AssertsCompiledSQL
+):
     def _roundtrip(
         self,
         Employee,
@@ -1488,6 +1491,63 @@ class ConcreteInhTest(_RemoveListeners, DeclarativeTestBase):
 
         self._roundtrip(Employee, Manager, Engineer, Boss, polymorphic=False)
 
+    def test_abstract_concrete_base_didnt_configure(self):
+        class Employee(AbstractConcreteBase, Base, fixtures.ComparableEntity):
+            pass
+
+        assert_raises_message(
+            orm_exc.UnmappedClassError,
+            "Class test.ext.declarative.test_inheritance.Employee is a "
+            "subclass of AbstractConcreteBase and has a mapping pending "
+            "until all subclasses are defined. Call the "
+            r"sqlalchemy.orm.configure_mappers\(\) function after all "
+            "subclasses have been defined to complete the "
+            "mapping of this class.",
+            Session().query,
+            Employee,
+        )
+
+        configure_mappers()
+
+        # no subclasses yet.
+        assert_raises_message(
+            orm_exc.UnmappedClassError,
+            ".*and has a mapping pending",
+            Session().query,
+            Employee,
+        )
+
+        class Manager(Employee):
+            __tablename__ = "manager"
+            employee_id = Column(
+                Integer, primary_key=True, test_needs_autoincrement=True
+            )
+            name = Column(String(50))
+            golf_swing = Column(String(40))
+            __mapper_args__ = {
+                "polymorphic_identity": "manager",
+                "concrete": True,
+            }
+
+        # didnt call configure_mappers() again
+        assert_raises_message(
+            orm_exc.UnmappedClassError,
+            ".*and has a mapping pending",
+            Session().query,
+            Employee,
+        )
+
+        configure_mappers()
+
+        self.assert_compile(
+            Session().query(Employee),
+            "SELECT pjoin.employee_id AS pjoin_employee_id, "
+            "pjoin.name AS pjoin_name, pjoin.golf_swing AS pjoin_golf_swing, "
+            "pjoin.type AS pjoin_type FROM (SELECT manager.employee_id "
+            "AS employee_id, manager.name AS name, manager.golf_swing AS "
+            "golf_swing, 'manager' AS type FROM manager) AS pjoin",
+        )
+
     def test_abstract_concrete_extension(self):
         class Employee(AbstractConcreteBase, Base, fixtures.ComparableEntity):
             pass
@@ -1901,8 +1961,8 @@ class ConcreteExtensionConfigTest(
             "(SELECT cca.id AS id, cca.x AS x, cca.y AS y, "
             "cca.something_id AS something_id, 'ccb' AS type FROM cca) "
             "AS pjoin WHERE EXISTS (SELECT 1 FROM something "
-            "WHERE something.id = pjoin.something_id AND "
-            "something.id = :id_1)",
+            "WHERE something.id = pjoin.something_id "
+            "AND something.id = :id_1)",
         )
 
         self.assert_compile(
@@ -1978,3 +2038,38 @@ class ConcreteExtensionConfigTest(
             "(SELECT offers.documenttype AS documenttype, offers.id AS id, "
             "'offer' AS type FROM offers) AS pjoin",
         )
+
+    def test_configure_discriminator_col(self):
+        """test #5513"""
+
+        class Employee(AbstractConcreteBase, Base):
+            _concrete_discriminator_name = "_alt_discriminator"
+            employee_id = Column(Integer, primary_key=True)
+
+        class Manager(Employee):
+            __tablename__ = "manager"
+
+            __mapper_args__ = {
+                "polymorphic_identity": "manager",
+                "concrete": True,
+            }
+
+        class Engineer(Employee):
+            __tablename__ = "engineer"
+
+            __mapper_args__ = {
+                "polymorphic_identity": "engineer",
+                "concrete": True,
+            }
+
+        configure_mappers()
+        self.assert_compile(
+            Session().query(Employee),
+            "SELECT pjoin.employee_id AS pjoin_employee_id, "
+            "pjoin._alt_discriminator AS pjoin__alt_discriminator "
+            "FROM (SELECT engineer.employee_id AS employee_id, "
+            "'engineer' AS _alt_discriminator FROM engineer "
+            "UNION ALL SELECT manager.employee_id AS employee_id, "
+            "'manager' AS _alt_discriminator "
+            "FROM manager) AS pjoin",
+        )
