@@ -1,8 +1,10 @@
+from copy import deepcopy
 import datetime
 import decimal
 
 from sqlalchemy import ARRAY
 from sqlalchemy import bindparam
+from sqlalchemy import Boolean
 from sqlalchemy import Column
 from sqlalchemy import Date
 from sqlalchemy import DateTime
@@ -27,18 +29,22 @@ from sqlalchemy.dialects import postgresql
 from sqlalchemy.dialects import sqlite
 from sqlalchemy.sql import column
 from sqlalchemy.sql import functions
+from sqlalchemy.sql import quoted_name
 from sqlalchemy.sql import table
 from sqlalchemy.sql.compiler import BIND_TEMPLATES
 from sqlalchemy.sql.functions import FunctionElement
 from sqlalchemy.sql.functions import GenericFunction
+from sqlalchemy.testing import assert_raises
 from sqlalchemy.testing import assert_raises_message
 from sqlalchemy.testing import AssertsCompiledSQL
 from sqlalchemy.testing import engines
 from sqlalchemy.testing import eq_
 from sqlalchemy.testing import fixtures
 from sqlalchemy.testing import is_
+from sqlalchemy.testing.assertions import expect_warnings
 from sqlalchemy.testing.engines import all_dialects
 
+
 table1 = table(
     "mytable",
     column("myid", Integer),
@@ -50,8 +56,15 @@ table1 = table(
 class CompileTest(fixtures.TestBase, AssertsCompiledSQL):
     __dialect__ = "default"
 
-    def tear_down(self):
-        functions._registry.clear()
+    def setup(self):
+        self._registry = deepcopy(functions._registry)
+        self._case_sensitive_registry = deepcopy(
+            functions._case_sensitive_registry
+        )
+
+    def teardown(self):
+        functions._registry = self._registry
+        functions._case_sensitive_registry = self._case_sensitive_registry
 
     def test_compile(self):
         for dialect in all_dialects(exclude=("sybase",)):
@@ -84,6 +97,9 @@ class CompileTest(fixtures.TestBase, AssertsCompiledSQL):
                 dialect=dialect,
             )
 
+            functions._registry["_default"].pop("fake_func")
+            functions._case_sensitive_registry["_default"].pop("fake_func")
+
     def test_use_labels(self):
         self.assert_compile(
             select([func.foo()], use_labels=True), "SELECT foo() AS foo_1"
@@ -92,6 +108,43 @@ class CompileTest(fixtures.TestBase, AssertsCompiledSQL):
     def test_underscores(self):
         self.assert_compile(func.if_(), "if()")
 
+    def test_underscores_packages(self):
+        self.assert_compile(func.foo_.bar_.if_(), "foo.bar.if()")
+
+    def test_uppercase(self):
+        # for now, we need to keep case insensitivity
+        self.assert_compile(func.UNREGISTERED_FN(), "UNREGISTERED_FN()")
+
+    def test_uppercase_packages(self):
+        # for now, we need to keep case insensitivity
+        self.assert_compile(func.FOO.BAR.NOW(), "FOO.BAR.NOW()")
+
+    def test_mixed_case(self):
+        # for now, we need to keep case insensitivity
+        self.assert_compile(func.SomeFunction(), "SomeFunction()")
+
+    def test_mixed_case_packages(self):
+        # for now, we need to keep case insensitivity
+        self.assert_compile(
+            func.Foo.Bar.SomeFunction(), "Foo.Bar.SomeFunction()"
+        )
+
+    def test_quote_special_chars(self):
+        # however we need to be quoting any other identifiers
+        self.assert_compile(
+            getattr(func, "im a function")(), '"im a function"()'
+        )
+
+    def test_quote_special_chars_packages(self):
+        # however we need to be quoting any other identifiers
+        self.assert_compile(
+            getattr(
+                getattr(getattr(func, "im foo package"), "im bar package"),
+                "im a function",
+            )(),
+            '"im foo package"."im bar package"."im a function"()',
+        )
+
     def test_generic_now(self):
         assert isinstance(func.now().type, sqltypes.DateTime)
 
@@ -163,12 +216,14 @@ class CompileTest(fixtures.TestBase, AssertsCompiledSQL):
             pass
 
         assert isinstance(func.myfunc(), myfunc)
+        self.assert_compile(func.myfunc(), "myfunc()")
 
     def test_custom_type(self):
         class myfunc(GenericFunction):
             type = DateTime
 
         assert isinstance(func.myfunc().type, DateTime)
+        self.assert_compile(func.myfunc(), "myfunc()")
 
     def test_custom_legacy_type(self):
         # in case someone was using this system
@@ -177,6 +232,38 @@ class CompileTest(fixtures.TestBase, AssertsCompiledSQL):
 
         assert isinstance(func.myfunc().type, DateTime)
 
+    def test_case_sensitive(self):
+        class MYFUNC(GenericFunction):
+            type = DateTime
+
+        assert isinstance(func.MYFUNC().type, DateTime)
+        assert isinstance(func.MyFunc().type, DateTime)
+        assert isinstance(func.mYfUnC().type, DateTime)
+        assert isinstance(func.myfunc().type, DateTime)
+
+    def test_replace_function(self):
+        class replaceable_func(GenericFunction):
+            type = Integer
+            identifier = "replaceable_func"
+
+        assert isinstance(func.Replaceable_Func().type, Integer)
+        assert isinstance(func.RePlAcEaBlE_fUnC().type, Integer)
+        assert isinstance(func.replaceable_func().type, Integer)
+
+        with expect_warnings(
+            "The GenericFunction 'replaceable_func' is already registered and "
+            "is going to be overridden.",
+            regex=False,
+        ):
+
+            class replaceable_func_override(GenericFunction):
+                type = DateTime
+                identifier = "replaceable_func"
+
+        assert isinstance(func.Replaceable_Func().type, DateTime)
+        assert isinstance(func.RePlAcEaBlE_fUnC().type, DateTime)
+        assert isinstance(func.replaceable_func().type, DateTime)
+
     def test_custom_w_custom_name(self):
         class myfunc(GenericFunction):
             name = "notmyfunc"
@@ -184,6 +271,21 @@ class CompileTest(fixtures.TestBase, AssertsCompiledSQL):
         assert isinstance(func.notmyfunc(), myfunc)
         assert not isinstance(func.myfunc(), myfunc)
 
+    def test_custom_w_quoted_name(self):
+        class myfunc(GenericFunction):
+            name = quoted_name("NotMyFunc", quote=True)
+            identifier = "myfunc"
+
+        self.assert_compile(func.myfunc(), '"NotMyFunc"()')
+
+    def test_custom_w_quoted_name_no_identifier(self):
+        class myfunc(GenericFunction):
+            name = quoted_name("NotMyFunc", quote=True)
+
+        # note this requires that the quoted name be lower cased for
+        # correct lookup
+        self.assert_compile(func.notmyfunc(), '"NotMyFunc"()')
+
     def test_custom_package_namespace(self):
         def cls1(pk_name):
             class myfunc(GenericFunction):
@@ -225,9 +327,19 @@ class CompileTest(fixtures.TestBase, AssertsCompiledSQL):
             type = Integer
             identifier = "buf3"
 
+        class GeoBufferFour(GenericFunction):
+            type = Integer
+            name = "BufferFour"
+            identifier = "Buf4"
+
         self.assert_compile(func.geo.buf1(), "BufferOne()")
         self.assert_compile(func.buf2(), "BufferTwo()")
         self.assert_compile(func.buf3(), "BufferThree()")
+        self.assert_compile(func.Buf4(), "BufferFour()")
+        self.assert_compile(func.BuF4(), "BufferFour()")
+        self.assert_compile(func.bUf4(), "BufferFour()")
+        self.assert_compile(func.bUf4_(), "BufferFour()")
+        self.assert_compile(func.buf4(), "BufferFour()")
 
     def test_custom_args(self):
         class myfunc(GenericFunction):
@@ -248,24 +360,13 @@ class CompileTest(fixtures.TestBase, AssertsCompiledSQL):
         c = column("abc")
         self.assert_compile(func.count(c), "count(abc)")
 
-    def test_constructor(self):
-        try:
-            func.current_timestamp("somearg")
-            assert False
-        except TypeError:
-            assert True
+    def test_ansi_functions_with_args(self):
+        ct = func.current_timestamp("somearg")
+        self.assert_compile(ct, "CURRENT_TIMESTAMP(:current_timestamp_1)")
 
-        try:
-            func.char_length("a", "b")
-            assert False
-        except TypeError:
-            assert True
-
-        try:
-            func.char_length()
-            assert False
-        except TypeError:
-            assert True
+    def test_char_length_fixed_args(self):
+        assert_raises(TypeError, func.char_length, "a", "b")
+        assert_raises(TypeError, func.char_length)
 
     def test_return_type_detection(self):
 
@@ -454,6 +555,15 @@ class CompileTest(fixtures.TestBase, AssertsCompiledSQL):
             "mytable.myid > :myid_1)",
         )
 
+    def test_funcfilter_arrayagg_subscript(self):
+        num = column("q")
+        self.assert_compile(
+            func.array_agg(num).filter(num % 2 == 0)[1],
+            "(array_agg(q) FILTER (WHERE q %% %(q_1)s = "
+            "%(param_1)s))[%(param_2)s]",
+            dialect="postgresql",
+        )
+
     def test_funcfilter_label(self):
         self.assert_compile(
             select(
@@ -643,6 +753,87 @@ class CompileTest(fixtures.TestBase, AssertsCompiledSQL):
             MissingType().compile,
         )
 
+    def test_as_comparison(self):
+
+        fn = func.substring("foo", "foobar").as_comparison(1, 2)
+        is_(fn.type._type_affinity, Boolean)
+
+        self.assert_compile(
+            fn.left, ":substring_1", checkparams={"substring_1": "foo"}
+        )
+        self.assert_compile(
+            fn.right, ":substring_1", checkparams={"substring_1": "foobar"}
+        )
+
+        self.assert_compile(
+            fn,
+            "substring(:substring_1, :substring_2)",
+            checkparams={"substring_1": "foo", "substring_2": "foobar"},
+        )
+
+    def test_as_comparison_annotate(self):
+
+        fn = func.foobar("x", "y", "q", "p", "r").as_comparison(2, 5)
+
+        from sqlalchemy.sql import annotation
+
+        fn_annotated = annotation._deep_annotate(fn, {"token": "yes"})
+
+        eq_(fn.left._annotations, {})
+        eq_(fn_annotated.left._annotations, {"token": "yes"})
+
+    def test_as_comparison_many_argument(self):
+
+        fn = func.some_comparison("x", "y", "z", "p", "q", "r").as_comparison(
+            2, 5
+        )
+        is_(fn.type._type_affinity, Boolean)
+
+        self.assert_compile(
+            fn.left,
+            ":some_comparison_1",
+            checkparams={"some_comparison_1": "y"},
+        )
+        self.assert_compile(
+            fn.right,
+            ":some_comparison_1",
+            checkparams={"some_comparison_1": "q"},
+        )
+
+        from sqlalchemy.sql import visitors
+
+        fn_2 = visitors.cloned_traverse(fn, {}, {})
+        fn_2.right = literal_column("ABC")
+
+        self.assert_compile(
+            fn,
+            "some_comparison(:some_comparison_1, :some_comparison_2, "
+            ":some_comparison_3, "
+            ":some_comparison_4, :some_comparison_5, :some_comparison_6)",
+            checkparams={
+                "some_comparison_1": "x",
+                "some_comparison_2": "y",
+                "some_comparison_3": "z",
+                "some_comparison_4": "p",
+                "some_comparison_5": "q",
+                "some_comparison_6": "r",
+            },
+        )
+
+        self.assert_compile(
+            fn_2,
+            "some_comparison(:some_comparison_1, :some_comparison_2, "
+            ":some_comparison_3, "
+            ":some_comparison_4, ABC, :some_comparison_5)",
+            checkparams={
+                "some_comparison_1": "x",
+                "some_comparison_2": "y",
+                "some_comparison_3": "z",
+                "some_comparison_4": "p",
+                "some_comparison_5": "r",
+            },
+        )
+
 
 class ReturnTypeTest(AssertsCompiledSQL, fixtures.TestBase):
     def test_array_agg(self):
@@ -832,6 +1023,7 @@ class ExecuteTest(fixtures.TestBase):
         t2.update(
             values={t2.c.value: func.length("asfdaasdf"), t2.c.stuff: "foo"}
         ).execute()
+
         eq_(select([t2.c.value, t2.c.stuff]).execute().first(), (9, "foo"))
 
     @testing.fails_on_everything_except("postgresql")
@@ -892,3 +1084,43 @@ def exec_sorted(statement, *args, **kw):
     return sorted(
         [tuple(row) for row in statement.execute(*args, **kw).fetchall()]
     )
+
+
+class RegisterTest(fixtures.TestBase, AssertsCompiledSQL):
+    __dialect__ = "default"
+
+    def setup(self):
+        self._registry = deepcopy(functions._registry)
+
+    def teardown(self):
+        functions._registry = self._registry
+
+    def test_GenericFunction_is_registered(self):
+        assert "GenericFunction" not in functions._registry["_default"]
+
+    def test_register_function(self):
+
+        # test generic function registering
+        class registered_func(GenericFunction):
+            _register = True
+
+            def __init__(self, *args, **kwargs):
+                GenericFunction.__init__(self, *args, **kwargs)
+
+        class registered_func_child(registered_func):
+            type = sqltypes.Integer
+
+        assert "registered_func" in functions._registry["_default"]
+        assert isinstance(func.registered_func_child().type, Integer)
+
+        class not_registered_func(GenericFunction):
+            _register = False
+
+            def __init__(self, *args, **kwargs):
+                GenericFunction.__init__(self, *args, **kwargs)
+
+        class not_registered_func_child(not_registered_func):
+            type = sqltypes.Integer
+
+        assert "not_registered_func" not in functions._registry["_default"]
+        assert isinstance(func.not_registered_func_child().type, Integer)
